searchData={"items":[{"type":"module","title":"ChangesetHelpers","doc":"Provides a set of helpers to work with Changesets.","ref":"ChangesetHelpers.html"},{"type":"function","title":"ChangesetHelpers.add_error/4","doc":"Adds an error to the nested changeset.\n\n```elixir\naccount_changeset =\n  ChangesetHelpers.add_error(account_changeset, [:user, :articles, :error_key], \"Some error\")\n```","ref":"ChangesetHelpers.html#add_error/4"},{"type":"function","title":"ChangesetHelpers.change_assoc/2","doc":"","ref":"ChangesetHelpers.html#change_assoc/2"},{"type":"function","title":"ChangesetHelpers.change_assoc/3","doc":"Returns the nested association in a changeset. This function will first look into the changes and then fails back on\ndata wrapped in a changeset.\n\nChanges may be added to the given changeset through the third argument.\n\nA tuple is returned containing the root changeset, and the changeset of the association.\n\n```elixir\n{account_changeset, address_changeset} =\n  change_assoc(account_changeset, [:user, :config, :address], %{street: \"Foo street\"})\n```","ref":"ChangesetHelpers.html#change_assoc/3"},{"type":"function","title":"ChangesetHelpers.change_assoc/4","doc":"Returns the nested association in a changeset at the given index.\n\nA tuple is returned containing the root changeset, the changesets of the association and the changeset at the\nspecified index.\n\nSee `change_assoc(struct_or_changeset, keys, changes)`.\n```","ref":"ChangesetHelpers.html#change_assoc/4"},{"type":"function","title":"ChangesetHelpers.diff_field/3","doc":"This function allows checking if a given field is different between two changesets.\n\n```elixir\n{street_changed, street1, street2} =\n  diff_field(account_changeset, new_account_changeset, [:user, :config, :address, :street])\n```","ref":"ChangesetHelpers.html#diff_field/3"},{"type":"function","title":"ChangesetHelpers.do_has_change?/2","doc":"","ref":"ChangesetHelpers.html#do_has_change?/2"},{"type":"function","title":"ChangesetHelpers.fetch_change/2","doc":"Fetches a nested change from the given changeset.\n\nThis function only looks at the `:changes` field of the given `changeset` and returns `{:ok, value}` if the change is\npresent or `:error` if it's not.\n\n```elixir\n{:ok, street} =\n  ChangesetHelpers.fetch_change(account_changeset, [:user, :config, :address, :street])\n```","ref":"ChangesetHelpers.html#fetch_change/2"},{"type":"function","title":"ChangesetHelpers.fetch_change!/2","doc":"Same as `fetch_change/2` but returns the value or raises if the given nested key was not found.\n\n```elixir\nstreet = ChangesetHelpers.fetch_change!(account_changeset, [:user, :config, :address, :street])\n```","ref":"ChangesetHelpers.html#fetch_change!/2"},{"type":"function","title":"ChangesetHelpers.fetch_field/2","doc":"Fetches the given nested field from changes or from the data.\n\nWhile `fetch_change/2` only looks at the current changes to retrieve a value, this function looks at the changes and\nthen falls back on the data, finally returning `:error` if no value is available.\n\nFor relations, these functions will return the changeset original data with changes applied. To retrieve raw\nchangesets, please use `fetch_change/2`.\n\n```elixir\n{:changes, street} =\n  ChangesetHelpers.fetch_field(account_changeset, [:user, :config, :address, :street])\n```","ref":"ChangesetHelpers.html#fetch_field/2"},{"type":"function","title":"ChangesetHelpers.fetch_field!/2","doc":"Same as `fetch_field/2` but returns the value or raises if the given nested key was not found.\n\n```elixir\nstreet = ChangesetHelpers.fetch_field!(account_changeset, [:user, :config, :address, :street])\n```","ref":"ChangesetHelpers.html#fetch_field!/2"},{"type":"function","title":"ChangesetHelpers.field_fails_validation?/3","doc":"","ref":"ChangesetHelpers.html#field_fails_validation?/3"},{"type":"function","title":"ChangesetHelpers.field_violates_constraint?/3","doc":"","ref":"ChangesetHelpers.html#field_violates_constraint?/3"},{"type":"function","title":"ChangesetHelpers.has_change?/2","doc":"","ref":"ChangesetHelpers.html#has_change?/2"},{"type":"function","title":"ChangesetHelpers.put_assoc/3","doc":"Puts the given nested association in the changeset through a given list of field names.\n\n```elixir\nChangesetHelpers.put_assoc(account_changeset, [:user, :config, :address], address_changeset)\n```\n\nInstead of giving a Changeset or a schema as the third argument, a function may also be given receiving the nested\nChangeset(s) to be updated as argument.\n\n```elixir\nChangesetHelpers.put_assoc(account_changeset, [:user, :articles],\n  &(Enum.concat(&1, [%Article{} |> Ecto.Changeset.change()])))\n```\n\nIn the code above, we add a new empty Article to the articles association (typically done when we want to add a new\narticle to a form).","ref":"ChangesetHelpers.html#put_assoc/3"},{"type":"function","title":"ChangesetHelpers.put_assoc/4","doc":"Puts the given nested association in the changeset at the given index.\n\nSee `put_assoc(changeset, keys, value)`.","ref":"ChangesetHelpers.html#put_assoc/4"},{"type":"function","title":"ChangesetHelpers.update_assoc_changes/3","doc":"","ref":"ChangesetHelpers.html#update_assoc_changes/3"},{"type":"function","title":"ChangesetHelpers.validate_changes/4","doc":"Works like `Ecto.Changeset.validate_change/3` but may receive multiple fields.\n\nThe `validator` function receives as argument a keyword list, where the keys are the field\nnames and the values are the change for this field, or the data.any()\n\nIf one of the fields is `nil`, the `validator` function is not invoked.","ref":"ChangesetHelpers.html#validate_changes/4"},{"type":"function","title":"ChangesetHelpers.validate_comparison/5","doc":"Validates the result of the comparison of\n  * two fields (where at least one is a change) or\n  * a change and a value, where the value is an integer, a `Date`, a `Time`,\n    a `DateTime` or a `NaiveDateTime`.","ref":"ChangesetHelpers.html#validate_comparison/5"},{"type":"function","title":"Options - ChangesetHelpers.validate_comparison/5","doc":"* `:error_on_field` - specifies on which field to add the error, defaults to the first field\n  * `:message` - a customized message on failure","ref":"ChangesetHelpers.html#validate_comparison/5-options"},{"type":"function","title":"ChangesetHelpers.validate_list/4","doc":"Validates a list of values using the given validator.\n\n```elixir\nchangeset =\n  %Appointment{}\n  |> Appointment.changeset(%{days_of_week: [1, 3, 8]})\n  |> validate_list(:days_of_week, &Ecto.Changeset.validate_inclusion/3, [1..7])\n\nassert [days_of_week: {\"is invalid\", [validation: :list, index: 2, validator: :validate_inclusion]}] = changeset.errors\nassert [days_of_week: {:list, [validator: :validate_inclusion]}] = changeset.validations\n```","ref":"ChangesetHelpers.html#validate_list/4"},{"type":"function","title":"ChangesetHelpers.validate_not_changed/2","doc":"`validate_not_changed/3` checks if the specified attribute is present in the changeset's parameters.\nIf the attribute is present, an error is added to the changeset's `:errors` key.","ref":"ChangesetHelpers.html#validate_not_changed/2"},{"type":"extras","title":"ChangesetHelpers","doc":"# ChangesetHelpers\n\nThis library provides a set of helper functions to work with Ecto Changesets.\n\n### `validate_comparison(changeset, field1, operator, field2_or_value, opts)`\n\nValidates the result of the comparison of\n  * two fields (where at least one is a change) or\n  * a change and a value, where the value is an integer, a `Date`, a `Time`,\n    a `DateTime` or a `NaiveDateTime`.\n\n```elixir\nvalidate_comparison(changeset, :start_time, :lt, :end_time)\n\nassert [start_time: {\"must be less than 10:00:00\", [validation: :comparison]}] = changeset.errors\nassert [start_time: :comparison, end_time: :comparison] = changeset.validations\n```\n\n```elixir\nvalidate_comparison(appointment_changeset, :end_time, :lt, ~T[21:00:00])\n\nassert [end_time: {\"must be less than 21:00:00\", [validation: :comparison]}] = changeset.errors\nassert [end_time: :comparison] = changeset.validations\n```\n\n### `validate_not_changed(changeset, field)`\n\nChecks if the specified attribute is present in the changeset's parameters. If the attribute is present, an error is added to the changeset's `:errors` key.\n\n```elixir\nchanges = %{foo: 1, bar: 2}\n\nchangeset =\n  %Appointment{}\n  |> cast(changes, [:foo])\n  |> validate_not_changed([:bar])\n  |> validate_not_changed([:baz, :qux])\n\nrefute changeset.valid?\nassert [bar: {\"cannot be changed\", [validation: :not_present]}] = changeset.errors\n```\n\n### `validate_list(changeset, field, validation_fun, validation_fun_args)`\n\nValidates a list of values using the given validator.\n\n```elixir\nchangeset =\n  %Appointment{}\n  |> Appointment.changeset(%{days_of_week: [1, 3, 8]})\n  |> validate_list(:days_of_week, &Ecto.Changeset.validate_inclusion/3, [1..7])\n\nassert [days_of_week: {\"is invalid\", [validation: :list, index: 2, validator: :validate_inclusion]}] = changeset.errors\nassert [days_of_week: {:list, [validator: :validate_inclusion]}] = changeset.validations\n```\n\nAs the validator function is from the `Ecto.Changeset` module, you may just write `:validate_inclusion`.\n\n### `put_assoc(changeset, keys, value)`\n\nPuts the given nested association in the changeset through a given list of field names.\n\n```elixir\nChangesetHelpers.put_assoc(account_changeset, [:user, :config, :address], address_changeset)\n```\n\nInstead of giving a Changeset or a schema as the third argument, a function may also be given in order to modify the\nnested changeset in one go.\n\n```elixir\nChangesetHelpers.put_assoc(account_changeset, [:user, :articles],\n  &(Enum.concat(&1, [%Article{} |> Ecto.Changeset.change()])))\n```\n\nIn the code above, we change a new empty Article, and add the changeset into the articles association (typically done when we want to add a new\nrow of form inputs to add an entity into a form handling a nested collection of entities).\n\n### `put_assoc(changeset, keys, index, value)`\n\nPuts the given nested association in the changeset through a given list of field names, at the given index.\n\n### `change_assoc(struct_or_changeset, keys, changes \\\\ %{})`\n\nReturns the nested association in a changeset. This function will first look into the changes and then fails back on\ndata wrapped in a changeset.\n\nChanges may be added to the given changeset through the third argument.\n\nA tuple is returned containing the modified root changeset and the changeset of the association.\n\n```elixir\n{account_changeset, address_changeset} =\n  change_assoc(account_changeset, [:user, :user_config, :address], %{street: \"Foo street\"})\n```\n\n### `change_assoc(struct_or_changeset, keys, index, changes \\\\ %{})`\n\nReturns the nested association in a changeset at the given index.\n\n### `update_assoc_changes(changeset, keys, changes)`\n\nAdds changes to the changed association changesets.\n\n### `fetch_field(changeset, keys)`\n\nFetches the given nested field from changes or from the data.\n\n```elixir\n{:changes, street} =\n  ChangesetHelpers.fetch_field(account_changeset, [:user, :config, :address, :street])\n```\n\n### `fetch_field!(changeset, keys)`\n\nSame as `fetch_field/2` but returns the value or raises if the given nested key was not found.\n\n```elixir\nstreet = ChangesetHelpers.fetch_field!(account_changeset, [:user, :config, :address, :street])\n```\n\n### `fetch_change(changeset, keys)`\n\nFetches the given nested field from changes or from the data.\n\n```elixir\n{:ok, street} =\n  ChangesetHelpers.fetch_change(account_changeset, [:user, :config, :address, :street])\n```\n\n### `fetch_change!(changeset, keys)`\n\nSame as `fetch_change/2` but returns the value or raises if the given nested key was not found.\n\n```elixir\nstreet = ChangesetHelpers.fetch_change!(account_changeset, [:user, :config, :address, :street])\n```\n\n### `has_change?(changeset, keys)`\n\n```elixir\nChangesetHelpers.has_change?(account_changeset, [:user, :config, :address, :street])\n```\n\n### `diff_field(changeset1, changeset2, keys)`\n\nThis function allows checking if a given field is different in two changesets.\n\n```elixir\n{street_changed, street1, street2} =\n  diff_field(account_changeset, new_account_changeset, [:user, :user_config, :address, :street])\n```\n\n### `add_error(changeset, keys, message, extra \\\\ [])`\n\nAdds an error to the nested changeset.\n\n```elixir\nChangesetHelpers.add_error(account_changeset, [:user, :articles, :error_key], \"Some error\")\n```\n\n### `field_fails_validation?(changeset, field, validations)`\n\nChecks whether a field as the given validation error key.\n\n```elixir\nChangesetHelpers.field_fails_validation?(changeset, :email, :unsafe_unique)\n```\n\n### `field_violates_constraint?(changeset, field, constraints)`\n\nChecks whether a field as the given constraint error key.\n\n```elixir\nChangesetHelpers.field_violates_constraint?(changeset, :email, :unique)\n```\n\n### `validate_changes(changeset, fields, meta, validator)`\n\nWorks like `Ecto.Changeset.validate_change/3` but may receive multiple fields.","ref":"readme.html"},{"type":"extras","title":"Installation - ChangesetHelpers","doc":"Add `changeset_helpers` for Elixir as a dependency in your `mix.exs` file:\n\n```elixir\ndef deps do\n  [\n    {:changeset_helpers, \"~> 0.23\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"HexDocs - ChangesetHelpers","doc":"HexDocs documentation can be found at [https://hexdocs.pm/changeset_helpers](https://hexdocs.pm/changeset_helpers).","ref":"readme.html#hexdocs"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}